@@ -20,11 +20,7 @@
  */
 
 #include "config.h"
-
 #include <string.h>
-
-#include <telepathy-glib/telepathy-glib.h>
-
 #include "mcd-account-manager-default.h"
 #include "mcd-debug.h"
 
@@ -290,57 +286,6 @@ _get_secrets_from_keyring (const McpAccountStorage *self,
                 g_warning ("Unsupported value type for %s.%s", account, name);
             }
 
-          if (!tp_strdiff (param, "password"))
-            {
-              /* Empathy 3.0 was meant to migrate passwords from MC to
-               * itself, but it couldn't complete the migration by
-               * deleting the password from MC, because MC had several
-               * bugs that meant deleting passwords didn't work. To atone
-               * for our past sins, detect an incomplete migration and
-               * complete it. */
-              GnomeKeyringResult empathy_ok =
-                GNOME_KEYRING_RESULT_NO_KEYRING_DAEMON;
-              GnomeKeyringAttributeList *empathy_match =
-                gnome_keyring_attribute_list_new ();
-              GList *empathy_items = NULL;
-
-              gnome_keyring_attribute_list_append_string (empathy_match,
-                  "account-id", account);
-              gnome_keyring_attribute_list_append_string (empathy_match,
-                  "param-name", "password");
-
-              empathy_ok = gnome_keyring_find_items_sync (
-                  GNOME_KEYRING_ITEM_GENERIC_SECRET, empathy_match,
-                  &empathy_items);
-
-              if (empathy_ok == GNOME_KEYRING_RESULT_OK &&
-                  empathy_items != NULL)
-                {
-                  KeyringSetData *ksd = g_slice_new0 (KeyringSetData);
-
-                  DEBUG ("An Empathy 3.0 password migration wasn't finished "
-                      "due to fd.o #42088. Finishing it now by deleting the "
-                      "password for %s", account);
-
-                  ksd->account = g_strdup (account);
-                  ksd->name = g_strdup ("password");
-                  ksd->set = FALSE;
-
-                  gnome_keyring_delete_password (&keyring_schema,
-                      _keyring_set_cb, ksd, NULL,
-                      "account", account,
-                      "param", "password",
-                      NULL);
-                }
-
-              gnome_keyring_found_list_free (empathy_items);
-
-              /* behave as if it had already been deleted, i.e. we never
-               * actually found it... */
-              param = NULL;
-              value = NULL;
-            }
-
           if (param != NULL && value != NULL)
             {
               gchar *key = g_strdup_printf ("param-%s", param);
@@ -579,6 +524,25 @@ _get (const McpAccountStorage *self,
   return TRUE;
 }
 
+static gchar *
+_create (const McpAccountStorage *self,
+    const McpAccountManager *am,
+    const gchar *manager,
+    const gchar *protocol,
+    GHashTable *params,
+    GError **error)
+{
+  gchar *unique_name;
+
+  /* See comment in plugin-account.c::_storage_create_account() before changing
+   * this implementation, it's more subtle than it looks */
+  unique_name = mcp_account_manager_get_unique_name (MCP_ACCOUNT_MANAGER (am),
+                                                     manager, protocol, params);
+  g_return_val_if_fail (unique_name != NULL, NULL);
+
+  return unique_name;
+}
+
 static gboolean
 _delete (const McpAccountStorage *self,
       const McpAccountManager *am,
@@ -701,6 +665,7 @@ account_storage_iface_init (McpAccountStorageIface *iface,
 
   mcp_account_storage_iface_implement_get (iface, _get);
   mcp_account_storage_iface_implement_set (iface, _set);
+  mcp_account_storage_iface_implement_create (iface, _create);
   mcp_account_storage_iface_implement_delete (iface, _delete);
   mcp_account_storage_iface_implement_commit_one (iface, _commit);
   mcp_account_storage_iface_implement_list (iface, _list);
