@@ -33,6 +33,8 @@
  * FIXME
  */
 
+#include "config.h"
+
 #include "mcd-connection.h"
 #include "mcd-connection-service-points.h"
 
@@ -47,25 +49,20 @@
 #include <stdlib.h>
 #include <dlfcn.h>
 
-#include <telepathy-glib/dbus.h>
-#include <telepathy-glib/interfaces.h>
-#include <telepathy-glib/gtypes.h>
-#include <telepathy-glib/connection.h>
+#include <telepathy-glib/telepathy-glib.h>
+#include <telepathy-glib/telepathy-glib-dbus.h>
+
 #include <telepathy-glib/proxy-subclass.h>
-#include <telepathy-glib/util.h>
 
 #include "mcd-account-priv.h"
 #include "mcd-channel-priv.h"
 #include "mcd-connection-priv.h"
 #include "mcd-dispatcher-priv.h"
 #include "mcd-channel.h"
-#include "mcd-provisioning-factory.h"
 #include "mcd-misc.h"
 #include "mcd-slacker.h"
 #include "sp_timestamp.h"
 
-#include "mcd-signals-marshal.h"
-
 #define INITIAL_RECONNECTION_TIME   3 /* seconds */
 #define RECONNECTION_MULTIPLIER     3
 #define MAXIMUM_RECONNECTION_TIME   30 * 60 /* half an hour */
@@ -140,7 +137,7 @@ struct _McdConnectionPrivate
 
     struct
     {
-        TpIntSet *handles;
+        TpIntset *handles;
         GSList *numbers;
     } emergency;
 };
@@ -576,9 +573,8 @@ on_new_channel (TpConnection *proxy, const gchar *chan_obj_path,
          * AddDispatchOperation or HandleChannels.
          *
          * We assume that channels without suppress_handler are incoming. */
-        _mcd_dispatcher_take_channels (priv->dispatcher,
-                                       g_list_prepend (NULL, channel),
-                                       suppress_handler, suppress_handler);
+        _mcd_dispatcher_add_channel (priv->dispatcher, channel,
+                                     suppress_handler, suppress_handler);
     }
 }
 
@@ -591,55 +587,6 @@ _foreach_channel_remove (McdMission * mission, McdOperation * operation)
     mcd_operation_remove_mission (operation, mission);
 }
 
-static void
-capabilities_advertise_cb (TpConnection *proxy, const GPtrArray *out0,
-			   const GError *error, gpointer user_data,
-			   GObject *weak_object)
-{
-    if (error)
-    {
-	g_warning ("%s: AdvertiseCapabilities failed: %s", G_STRFUNC, error->message);
-    }
-    
-}
-
-static void
-_mcd_connection_setup_capabilities (McdConnection *connection)
-{
-    McdConnectionPrivate *priv = MCD_CONNECTION_PRIV (connection);
-    GPtrArray *capabilities;
-    const gchar *removed = NULL;
-    GType type;
-    guint i;
-
-    if (priv->has_contact_capabilities_if)
-    {
-        DEBUG ("ContactCapabilities in use, avoiding Capabilities");
-        return;
-    }
-
-    if (!priv->has_capabilities_if)
-    {
-        DEBUG ("connection does not support capabilities interface");
-	return;
-    }
-    capabilities = _mcd_dispatcher_get_channel_capabilities (priv->dispatcher);
-    DEBUG ("advertising capabilities");
-    tp_cli_connection_interface_capabilities_call_advertise_capabilities (priv->tp_conn, -1,
-									  capabilities,
-									  &removed,
-									  capabilities_advertise_cb,
-									  priv, NULL,
-									  (GObject *) connection);
-
-    /* free the connection capabilities */
-    type = dbus_g_type_get_struct ("GValueArray", G_TYPE_STRING,
-				   G_TYPE_UINT, G_TYPE_INVALID);
-    for (i = 0; i < capabilities->len; i++)
-	g_boxed_free (type, g_ptr_array_index (capabilities, i));
-    g_ptr_array_unref (capabilities);
-}
-
 static void
 avatars_set_avatar_cb (TpConnection *proxy, const gchar *token,
 		       const GError *error, gpointer user_data,
@@ -1223,6 +1170,10 @@ connect_cb (TpConnection *tp_conn, const GError *error,
     }
 }
 
+static gboolean
+_mcd_connection_request_channel (McdConnection *connection,
+                                 McdChannel *channel);
+
 static void
 request_unrequested_channels (McdConnection *connection)
 {
@@ -1268,16 +1219,16 @@ mcd_connection_find_channel_by_path (McdConnection *connection,
     return NULL;
 }
 
+static gboolean mcd_connection_need_dispatch (McdConnection *connection,
+                                              const gchar *object_path,
+                                              GHashTable *props);
+
 static void
 on_new_channels (TpConnection *proxy, const GPtrArray *channels,
                  gpointer user_data, GObject *weak_object)
 {
     McdConnection *connection = MCD_CONNECTION (weak_object);
     McdConnectionPrivate *priv = user_data;
-    McdChannel *channel;
-    GList *channel_list = NULL;
-    gboolean requested = FALSE;
-    gboolean only_observe = FALSE;
     guint i;
 
     if (DEBUGGING)
@@ -1308,9 +1259,6 @@ on_new_channels (TpConnection *proxy, const GPtrArray *channels,
      * FALSE: they'll also be in Channels in the GetAll(Requests) result */
     if (!priv->dispatched_initial_channels) return;
 
-    only_observe = ! MCD_CONNECTION_GET_CLASS (connection)->need_dispatch (
-        connection, channels);
-
     sp_timestamp ("NewChannels received");
     for (i = 0; i < channels->len; i++)
     {
@@ -1318,11 +1266,17 @@ on_new_channels (TpConnection *proxy, const GPtrArray *channels,
         const gchar *object_path;
         GHashTable *props;
         GValue *value;
+        gboolean requested = FALSE;
+        gboolean only_observe = FALSE;
+        McdChannel *channel;
 
         va = g_ptr_array_index (channels, i);
         object_path = g_value_get_boxed (va->values);
         props = g_value_get_boxed (va->values + 1);
 
+        only_observe = !mcd_connection_need_dispatch (connection, object_path,
+                                                      props);
+
         /* Don't do anything for requested channels */
         value = g_hash_table_lookup (props, TP_IFACE_CHANNEL ".Requested");
         if (value && g_value_get_boolean (value))
@@ -1341,17 +1295,15 @@ on_new_channels (TpConnection *proxy, const GPtrArray *channels,
                                         MCD_MISSION (channel));
         }
 
-        channel_list = g_list_prepend (channel_list, channel);
-    }
+        if (!requested)
+        {
+            /* we always dispatch unrequested (incoming) channels */
+            only_observe = FALSE;
+        }
 
-    if (!requested)
-    {
-        /* we always dispatch unrequested (incoming) channels */
-        only_observe = FALSE;
+        _mcd_dispatcher_add_channel (priv->dispatcher, channel, requested,
+                                     only_observe);
     }
-
-    _mcd_dispatcher_take_channels (priv->dispatcher, channel_list, requested,
-                                   only_observe);
 }
 
 static void
@@ -1561,21 +1513,25 @@ mcd_connection_setup_pre_requests (McdConnection *connection)
 }
 
 static void
-on_connection_ready (TpConnection *tp_conn, const GError *error,
-		     gpointer user_data)
+on_connection_ready (GObject *source_object, GAsyncResult *result,
+                     gpointer user_data)
 {
+    TpConnection *tp_conn = TP_CONNECTION (source_object);
     McdConnection *connection, **connection_ptr = user_data;
     McdConnectionPrivate *priv;
+    GError *error = NULL;
 
     connection = *connection_ptr;
     if (connection)
 	g_object_remove_weak_pointer ((GObject *)connection,
 				      (gpointer)connection_ptr);
     g_slice_free (McdConnection *, connection_ptr);
-    if (error)
+
+    if (!tp_proxy_prepare_finish (tp_conn, result, &error))
     {
         DEBUG ("got error: %s", error->message);
-	return;
+        g_clear_error (&error);
+        return;
     }
 
     if (!connection) return;
@@ -1599,9 +1555,6 @@ on_connection_ready (TpConnection *tp_conn, const GError *error,
     if (priv->has_presence_if)
 	_mcd_connection_setup_presence (connection);
 
-    if (priv->has_capabilities_if)
-	_mcd_connection_setup_capabilities (connection);
-
     if (priv->has_avatars_if)
 	_mcd_connection_setup_avatar (connection);
 
@@ -2176,18 +2129,19 @@ _mcd_connection_get_property (GObject * obj, guint prop_id,
 /*
  * mcd_connection_need_dispatch:
  * @connection: the #McdConnection.
- * @channels: array of #McdChannel elements.
+ * @object_path: the object path of the new channel (only for debugging)
+ * @props: the properties of the new channel
  *
  * This functions must be called in response to a NewChannels signals, and is
  * responsible for deciding whether MC must handle the channels or not.
  */
 static gboolean
 mcd_connection_need_dispatch (McdConnection *connection,
-                              const GPtrArray *channels)
+                              const gchar *object_path,
+                              GHashTable *props)
 {
     McdAccount *account = mcd_connection_get_account (connection);
-    gboolean any_requested = FALSE, requested_by_us = FALSE;
-    guint i;
+    gboolean requested = FALSE, requested_by_us = FALSE;
 
     if (_mcd_account_needs_dispatch (account))
     {
@@ -2200,31 +2154,18 @@ mcd_connection_need_dispatch (McdConnection *connection,
      * have no McdChannel object associated: these are the channels directly
      * requested to the CM by some other application, and we must ignore them
      */
-    for (i = 0; i < channels->len; i++)
+
+    requested = tp_asv_get_boolean (props, TP_IFACE_CHANNEL ".Requested",
+                                    NULL);
+    if (requested)
     {
-        GValueArray *va;
-        const gchar *object_path;
-        GHashTable *props;
-        gboolean requested;
-
-        va = g_ptr_array_index (channels, i);
-        object_path = g_value_get_boxed (va->values);
-        props = g_value_get_boxed (va->values + 1);
-
-        requested = tp_asv_get_boolean (props, TP_IFACE_CHANNEL ".Requested",
-                                        NULL);
-        if (requested)
-        {
-            any_requested = TRUE;
-
-            if (mcd_connection_find_channel_by_path (connection, object_path))
-                requested_by_us = TRUE;
-        }
+        if (mcd_connection_find_channel_by_path (connection, object_path))
+            requested_by_us = TRUE;
     }
 
     /* handle only bundles which were not requested or that were requested
      * through MC */
-    return !any_requested || requested_by_us;
+    return !requested || requested_by_us;
 }
 
 gboolean
@@ -2255,32 +2196,9 @@ _mcd_connection_target_handle_is_urgent (McdConnection *self, guint handle)
   return FALSE;
 }
 
-gboolean
-_mcd_connection_channel_is_urgent (McdConnection *self, McdChannel *channel)
-{
-    const gchar *name = NULL;
-
-    if (mcd_channel_get_handle_type (channel) != TP_HANDLE_TYPE_CONTACT)
-        return FALSE;
-
-    name = mcd_channel_get_name (channel);
-
-    if (tp_str_empty (name))
-    {
-        TpHandle handle = mcd_channel_get_handle (channel);
-
-        return _mcd_connection_target_handle_is_urgent (self, handle);
-    }
-    else
-    {
-        return _mcd_connection_target_id_is_urgent (self, name);
-    }
-
-    return FALSE;
-}
-
 static gboolean
-_mcd_connection_request_channel (McdConnection *connection, McdChannel *channel)
+_mcd_connection_request_channel (McdConnection *connection,
+                                 McdChannel *channel)
 {
     McdConnectionPrivate *priv = MCD_CONNECTION_PRIV (connection);
     gboolean ret;
@@ -2288,7 +2206,7 @@ _mcd_connection_request_channel (McdConnection *connection, McdChannel *channel)
     g_return_val_if_fail (priv->tp_conn != NULL, FALSE);
     g_return_val_if_fail (TP_IS_CONNECTION (priv->tp_conn), FALSE);
 
-    if (!tp_connection_is_ready (priv->tp_conn))
+    if (!tp_proxy_is_prepared (priv->tp_conn, TP_CONNECTION_FEATURE_CONNECTED))
     {
         /* don't request any channel until the connection is ready (because we
          * don't know if the CM implements the Requests interface). The channel
@@ -2304,7 +2222,7 @@ _mcd_connection_request_channel (McdConnection *connection, McdChannel *channel)
     else
     {
         mcd_channel_take_error (channel,
-                                g_error_new (TP_ERRORS,
+                                g_error_new (TP_ERROR,
                                              TP_ERROR_NOT_IMPLEMENTED,
                                              "No Requests interface"));
         mcd_mission_abort ((McdMission *) channel);
@@ -2328,9 +2246,6 @@ mcd_connection_class_init (McdConnectionClass * klass)
     object_class->set_property = _mcd_connection_set_property;
     object_class->get_property = _mcd_connection_get_property;
 
-    klass->need_dispatch = mcd_connection_need_dispatch;
-    klass->request_channel = _mcd_connection_request_channel;
-
     _mcd_ext_register_dbus_glib_marshallers ();
 
     tp_connection_init_known_interfaces ();
@@ -2380,7 +2295,7 @@ mcd_connection_class_init (McdConnectionClass * klass)
     signals[SELF_PRESENCE_CHANGED] = g_signal_new ("self-presence-changed",
         G_OBJECT_CLASS_TYPE (klass),
         G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED, 0,
-        NULL, NULL, _mcd_marshal_VOID__UINT_STRING_STRING,
+        NULL, NULL, NULL,
         G_TYPE_NONE, 3, G_TYPE_UINT, G_TYPE_STRING, G_TYPE_STRING);
 
     signals[SELF_NICKNAME_CHANGED] = g_signal_new ("self-nickname-changed",
@@ -2392,7 +2307,7 @@ mcd_connection_class_init (McdConnectionClass * klass)
     signals[CONNECTION_STATUS_CHANGED] = g_signal_new (
         "connection-status-changed", G_OBJECT_CLASS_TYPE (klass),
         G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED, 0,
-        NULL, NULL, _mcd_marshal_VOID__UINT_UINT_OBJECT,
+        NULL, NULL, NULL,
         G_TYPE_NONE, 3, G_TYPE_UINT, G_TYPE_UINT, TP_TYPE_CONNECTION);
 
     signals[READY] = g_signal_new ("ready",
@@ -2554,46 +2469,7 @@ mcd_connection_request_channel (McdConnection *connection,
         mcd_operation_take_mission (MCD_OPERATION (connection),
                                     MCD_MISSION (channel));
 
-    return MCD_CONNECTION_GET_CLASS (connection)->request_channel (connection,
-                                                                   channel);
-}
-
-gboolean
-mcd_connection_cancel_channel_request (McdConnection *connection,
-				       guint operation_id,
-				       const gchar *requestor_client_id,
-				       GError **error)
-{
-    const GList *channels, *node;
-    McdChannel *channel;
-
-    /* first, see if the channel is in the list of the pending channels */
-
-    channels = mcd_operation_get_missions (MCD_OPERATION (connection));
-    if (!channels) return FALSE;
-
-    for (node = channels; node; node = node->next)
-    {
-	guint chan_requestor_serial;
-	gchar *chan_requestor_client_id;
-
-	channel = MCD_CHANNEL (node->data);
-	g_object_get (channel,
-		      "requestor-serial", &chan_requestor_serial,
-		      "requestor-client-id", &chan_requestor_client_id,
-		      NULL);
-	if (chan_requestor_serial == operation_id &&
-	    strcmp (chan_requestor_client_id, requestor_client_id) == 0)
-	{
-            DEBUG ("requested channel found (%p)", channel);
-	    mcd_mission_abort (MCD_MISSION (channel));
-	    g_free (chan_requestor_client_id);
-	    return TRUE;
-	}
-	g_free (chan_requestor_client_id);
-    }
-    DEBUG ("requested channel not found!");
-    return FALSE;
+    return _mcd_connection_request_channel (connection, channel);
 }
 
 void
@@ -2716,6 +2592,10 @@ _mcd_connection_set_tp_connection (McdConnection *connection,
 {
     McdConnection **connection_ptr;
     McdConnectionPrivate *priv;
+    GQuark features[] = {
+      TP_CONNECTION_FEATURE_CONNECTED,
+      0
+    };
 
     g_return_if_fail (MCD_IS_CONNECTION (connection));
     priv = connection->priv;
@@ -2763,8 +2643,8 @@ _mcd_connection_set_tp_connection (McdConnection *connection,
     *connection_ptr = connection;
     g_object_add_weak_pointer ((GObject *)connection,
                                (gpointer)connection_ptr);
-    tp_connection_call_when_ready (priv->tp_conn, on_connection_ready,
-                                   connection_ptr);
+    tp_proxy_prepare_async (priv->tp_conn, features,
+                            on_connection_ready, connection_ptr);
 }
 
 /**
@@ -2786,7 +2666,7 @@ _mcd_connection_is_ready (McdConnection *self)
     g_return_val_if_fail (MCD_IS_CONNECTION (self), FALSE);
 
     return (self->priv->tp_conn != NULL) &&
-        tp_connection_is_ready (self->priv->tp_conn);
+        tp_proxy_is_prepared (self->priv->tp_conn, TP_CONNECTION_FEATURE_CONNECTED);
 }
 
 gboolean
@@ -2799,30 +2679,6 @@ _mcd_connection_presence_info_is_ready (McdConnection *self)
 
 static void clear_emergency_handles (McdConnectionPrivate *priv)
 {
-  guint n_handles = 0;
-
-  if (priv->emergency.handles != NULL)
-    n_handles = tp_intset_size (priv->emergency.handles);
-
-  /* trawl through the handles and unref them */
-  if (n_handles > 0)
-    {
-      TpIntSetFastIter iter;
-      TpHandle *handles = g_new0 (TpHandle, n_handles);
-      TpHandle handle;
-      guint i = 0;
-
-      tp_intset_fast_iter_init (&iter, priv->emergency.handles);
-
-      while (tp_intset_fast_iter_next (&iter, &handle))
-        handles[i++] = handle;
-
-      tp_connection_unref_handles (priv->tp_conn, TP_HANDLE_TYPE_CONTACT,
-          n_handles, handles);
-
-      g_free (handles);
-    }
-
   tp_clear_pointer (&priv->emergency.handles, tp_intset_destroy);
 }
 
@@ -2852,7 +2708,7 @@ void _mcd_connection_take_emergency_numbers (McdConnection *self, GSList *number
 }
 
 void
-_mcd_connection_take_emergency_handles (McdConnection *self, TpIntSet *handles)
+_mcd_connection_take_emergency_handles (McdConnection *self, TpIntset *handles)
 {
     McdConnectionPrivate *priv = self->priv;
 
