@@ -27,8 +27,8 @@
 #ifndef __MCD_DBUSPROP_H__
 #define __MCD_DBUSPROP_H__
 
-#include <telepathy-glib/svc-generic.h>
-#include <telepathy-glib/dbus.h>
+#include <telepathy-glib/telepathy-glib.h>
+#include <telepathy-glib/telepathy-glib-dbus.h>
 
 G_BEGIN_DECLS
 
@@ -49,7 +49,6 @@ typedef struct _McdDBusProp
     const gchar *name;
     mcd_setprop setprop;
     mcd_getprop getprop;
-    mcd_async_getprop async_getprop;
 } McdDBusProp;
 
 typedef struct _McdInterfaceData
@@ -59,6 +58,7 @@ typedef struct _McdInterfaceData
     const McdDBusProp *properties;
     GInterfaceInitFunc iface_init;
     McdInterfaceInit instance_init;
+    gboolean optional;
 } McdInterfaceData;
 
 #define MCD_IMPLEMENT_IFACE(type, type_name, dbus_name) \
@@ -68,6 +68,7 @@ typedef struct _McdInterfaceData
     type_name##_properties, \
     (GInterfaceInitFunc)type_name##_iface_init, \
     NULL, \
+    FALSE, \
 }
 
 #define MCD_IMPLEMENT_IFACE_WITH_INIT(type, type_name, dbus_name) \
@@ -77,6 +78,27 @@ typedef struct _McdInterfaceData
     type_name##_properties, \
     (GInterfaceInitFunc)type_name##_iface_init, \
     type_name##_instance_init, \
+    FALSE, \
+}
+
+#define MCD_IMPLEMENT_OPTIONAL_IFACE(type, type_name, dbus_name) \
+{ \
+    type, \
+    dbus_name, \
+    type_name##_properties, \
+    (GInterfaceInitFunc)type_name##_iface_init, \
+    NULL, \
+    TRUE, \
+}
+
+#define MCD_IMPLEMENT_OPTIONAL_IFACE_WITH_INIT(type, type_name, dbus_name) \
+{ \
+    type, \
+    dbus_name, \
+    type_name##_properties, \
+    (GInterfaceInitFunc)type_name##_iface_init, \
+    type_name##_instance_init, \
+    TRUE, \
 }
 
 void mcd_dbus_init_interfaces (GType g_define_type_id,
@@ -144,5 +166,10 @@ void mcd_dbus_get_interfaces (TpSvcDBusProperties *self,
 			      const gchar *name,
 			      GValue *value);
 
+void mcd_dbus_activate_optional_interface (TpSvcDBusProperties *object,
+    GType interface);
+gboolean mcd_dbus_is_active_optional_interface (TpSvcDBusProperties *object,
+    GType interface);
+
 G_END_DECLS
 #endif /* __MCD_DBUSPROP_H__ */
