@@ -24,8 +24,7 @@
 #ifndef __MCD_DISPATCH_OPERATION_PRIV_H__
 #define __MCD_DISPATCH_OPERATION_PRIV_H__
 
-#include <telepathy-glib/dbus.h>
-#include <telepathy-glib/enums.h>
+#include <telepathy-glib/telepathy-glib.h>
 
 #include "client-registry.h"
 #include "mcd-handler-map-priv.h"
@@ -70,15 +69,18 @@ G_GNUC_INTERNAL void _mcd_dispatch_operation_approve
 #define MCD_DISPATCH_OPERATION_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), MCD_TYPE_DISPATCH_OPERATION, McdDispatchOperationClass))
 
 G_GNUC_INTERNAL McdDispatchOperation *_mcd_dispatch_operation_new (
-    McdClientRegistry *client_registry, McdHandlerMap *handler_map,
-    gboolean needs_approval, gboolean observe_only, GList *channels,
+    McdClientRegistry *client_registry,
+    McdHandlerMap *handler_map,
+    gboolean needs_approval,
+    gboolean observe_only,
+    McdChannel *channel,
     const gchar * const *possible_handlers);
 
 G_GNUC_INTERNAL gboolean _mcd_dispatch_operation_has_channel (
     McdDispatchOperation *self, McdChannel *channel);
-G_GNUC_INTERNAL const GList *_mcd_dispatch_operation_peek_channels (
+G_GNUC_INTERNAL McdChannel *_mcd_dispatch_operation_peek_channel (
     McdDispatchOperation *self);
-G_GNUC_INTERNAL GList *_mcd_dispatch_operation_dup_channels (
+G_GNUC_INTERNAL McdChannel *_mcd_dispatch_operation_dup_channel (
     McdDispatchOperation *self);
 
 G_GNUC_INTERNAL gboolean _mcd_dispatch_operation_is_finished (
