@@ -29,8 +29,9 @@
 
 #include <glib.h>
 #include <glib-object.h>
-#include <telepathy-glib/dbus.h>
-#include <telepathy-glib/gtypes.h>
+
+#include <telepathy-glib/telepathy-glib.h>
+#include <telepathy-glib/telepathy-glib-dbus.h>
 
 G_BEGIN_DECLS
 
@@ -48,16 +49,9 @@ void _mcd_object_call_on_struct_when_ready (gpointer object, gpointer strukt,
 void _mcd_object_ready (gpointer object, GQuark quark, const GError *error);
 
 G_GNUC_INTERNAL
-gboolean _mcd_file_set_contents (const gchar *filename, const gchar *contents,
-                                 gssize length, GError **error);
-G_GNUC_INTERNAL
-void _mc_ext_register_dbus_glib_marshallers (void);
+void _mcd_ext_register_dbus_glib_marshallers (void);
 
 G_GNUC_INTERNAL int _mcd_chmod_private (const gchar *filename);
 
-G_GNUC_INTERNAL GError *_mcd_g_error_new_valist (GQuark domain, gint code,
-                                                 const gchar *format,
-                                                 va_list ap);
-
 G_END_DECLS
 #endif /* MCD_MISC_H */
